/**
 * ELIXON THEATRE
 * Project Homepage: http://www.webdevelopers.eu/jquery/theatre
 *
 * LICENSE
 *   http://www.webdevelopers.eu/jquery/theatre/license
 *   Commons Attribution-NonCommercial 3.0
 *
 *   Get a commercial license at
 *   http://www.webdevelopers.eu/jquery/theatre/buy 
 *
 * DEMO
 *    http://www.webdevelopers.eu/jquery/theatre/demo 
 * 
 * DOCUMENTATION
 *    http://www.webdevelopers.eu/jquery/theatre/documentation
 *    
 * @project    Elixon CMS, http://www.webdevelopers.eu/
 * @package    JQuery
 * @subpackage Theatre
 * @author     Daniel Sevcik <sevcik@webdevelopers.eu>
 * @version    1.0.4-beta
 * @copyright  2010 Daniel Sevcik
 * @since      2010-11-11T22:10:07+0100
 * @revision   jQueryRevision: 4652 jQuery
 * @changed    jQueryDate: 2010-12-01 11:35:11 +0100 (Wed, 01 Dec 2010) jQuery
 * @access     public
 */
(function(jQuery) {
	var effects={};
	var methods={};
	var urlBase;

	jQuery.fn.theatre=function() {
		var currArguments=arguments;
		(this.length ? this : jQuery(document)).each(function() {
				methods.initAll.apply(jQuery(this), currArguments);
			});
		return this;
	}

	// name:string - 'FILE:EFFECT' or 'EFFECT', one file may register EFFECT or multiple effect prefixed by FILE string
	methods.loadEffect=function(name) { 
		if (!urlBase) { // Find the locations
			jQuery('script[src*="jquery.theatre-"], link[href*="theatre.css"]').first().each(function() {
					urlBase=(jQuery(this).attr('href') || jQuery(this).attr('src')).replace(/\/[^\/]*(#.*)?jQuery/, '');
				});
		}
		var url=urlBase+'/effect.'+name.split(':')[0]+'.js';
		jQuery('head').append('<script type="text/javascript" src="'+url+'"></script>');
		return effects[name];
	}

	methods.initAll=function(method) {
		// Init theatre
		if (typeof method == 'object' || !method || method == 'init') { 
			return methods.init.apply(this, arguments);
		}
		
		// Register Effect
		if (method == 'effect') {
			// Register new effect
			if (typeof arguments[2] == 'function') {
				return effects[arguments[1]]=arguments[2];
			} else {
				jQuery.error("Elixon Theatre cannot register effect object unless it is a Function.");
			}
		}

		var theatre=this.data('theatre');
		if (!theatre) return false; // not initialized yet
		
		switch(method) {
		case 'jump':
		    var jumpTo;
			switch(jumpTo) {
			case 'first': jumpTo=0; break;
			case 'last' : jumpTo=theatre.actors.length - 1; break;
			default     :
				jumpTo=(parseInt(arguments[1]) - 1) % theatre.actors.length;
				jumpTo=(Math.abs(Math.floor(jumpTo / theatre.actors.length) * theatre.actors.length) + jumpTo) % theatre.actors.length; //normalize				
			}
			while(theatre.index != jumpTo) {
			  if (theatre.effect.jump) {
				  theatre.effect.jump(jumpTo);
				  theatre.index=jumpTo;
			  } else {
				  this.theatre(theatre.index < jumpTo ? 'next' : 'prev');
			  }
			}
		    break;
		case 'next':
		case 'prev':		
			// Call next/prev custom effects
			if (!arguments[1]) this.theatre('stop'); // stop animation if no second param == true
		    theatre.index=(theatre.index + (method == 'next' ? 1 : -1)) % theatre.actors.length;
			theatre.index=(Math.abs(Math.floor(theatre.index / theatre.actors.length) * theatre.actors.length) + theatre.index) % theatre.actors.length; //normalize
		    if (typeof theatre.settings.onMove == 'function') theatre.settings.onMove.apply(this, [theatre.index]);
		    theatre.effect[method].apply(theatre.effect, Array.prototype.slice.call(arguments, 1));
			break;
		case 'play':
		case 'stop':		
			methods[method].apply(this, arguments);
		    break;
		default: // Unsupported
			jQuery.error('Elixon Theatre method "'+method+'" does not exist on jQuery.theatre!');
		}
	}

	methods.init=function(options) {
		methods.destroy.apply(this); // Reset old if any
		
		// Default settings
		var settings={
			selector: '> *:not(".theatre-control")',
			effect: 'horizontal', // 'horizontal'|'vertical'|'fade'|'show'|'slide'|'3d'|CUSTOM EFFECT NAME|OBJECT with constructor implementing the init,next,prev methods
			speed: 1000, // Transition speed
			still: 3000, // Time between transitions
			autoplay: true,
			controls: 'horizontal', // display control buttons 'horizontal' or 'vertical' or 'none'
			itemWidth: false, // width of the item or 'max' or false. 
			itemHeight: false, // height of the item or 'max' or false. 
			width: false, // set the height of the container. By default honors the already set size using CSS
			height: false, // set the height of the container. By default honors the already set size using CSS
			onMove: false // function(index) {}, execution context is the Stage
		}
		if (options) {
			jQuery.extend(settings, options);
		}

		// Init effect object
		var actors=jQuery(settings.selector, this);
		var theatre={actors: actors, effect: false, settings: settings, interval: false, index: 0};
		var effect=(typeof settings.effect == 'function' ? settings.effect : effects[settings.effect] || methods.loadEffect(settings.effect));
		if (!effect) jQuery.error('Elixon Theatre does not support effect "'+settings.effect+'"!');
		theatre.effect=new effects[settings.effect](this, actors, settings, theatre);
		
		// Stage
		this.addClass('theatre').data('theatre', theatre);
		this.addClass('theatre-'+settings.effect.replace(/[^a-z0-9]+/ig, '-'));
		if (settings.width) this.css('width', settings.width);
		if (settings.height) this.css('height', settings.height);
		
		// Actors
		actors.each(function(){if (!jQuery(this).data('theatre')) jQuery(this).data('theatre', {width: jQuery(this).width(), height: jQuery(this).height()});});		
		if (settings.itemWidth || settings.itemHeight) {
			var thisObj=this;
			actors.each(function() {
					if (settings.itemWidth) {
						jQuery(this).css('width', settings.itemWidth == 'max' ? (thisObj.width() - jQuery(this).outerWidth() + jQuery(this).width())+'px' : settings.itemWidth);
					}
					if (settings.itemHeight) {
						jQuery(this).css('height', settings.itemHeight == 'max' ? (thisObj.height() - jQuery(this).outerHeight() + jQuery(this).height())+'px' : settings.itemHeight);
					}
				});
		}
		
		actors.addClass('theatre-actor').stop(true, true);
		theatre.effect.init();

		if (settings.autoplay) {
			methods.play.apply(this);
		}

		// Controls
		methods.appendControls.apply(this); 
	}

	methods.appendControls=function() {
		settings=this.data('theatre').settings;
		// Controls
		if (settings.controls == 'horizontal' || settings.controls == 'vertical') {
			var thisObj=this;
			this.append('<a class="theatre-control theatre-control-'+settings.controls+'-next theatre-next"><span></span></a>');
			this.append('<a class="theatre-control theatre-control-'+settings.controls+'-prev theatre-prev"><span></span></a>');
			//this.append('<a class="theatre-control theatre-control-'+settings.controls+'-play theatre-play"><span></span></a>');
			//this.append('<a class="theatre-control theatre-control-'+settings.controls+'-stop theatre-stop"><span></span></a>');
			jQuery('.theatre-next', this).click(function() {thisObj.theatre('next');});
			jQuery('.theatre-prev', this).click(function() {thisObj.theatre('prev');});
			//jQuery('.theatre-play', this).click(function() {thisObj.theatre('play');});
			//jQuery('.theatre-stop', this).click(function() {thisObj.theatre('stop');});
			this.mouseenter(function() {jQuery('.theatre-control', thisObj).fadeIn();});
			this.mouseleave(function() {jQuery('.theatre-control', thisObj).fadeOut();});
			jQuery('.theatre-control', this).fadeOut(0);
		}

		this.append('<a class="theatre-control theatre-sign" rel="copyright license" style="position: absolute !important; display: none !important;" href="http://www.webdevelopers.eu/jquery/theatre" title="jQuery carousel plugin"><span style="display: none !important;">Elixon Theatre jQuery Plugin</span></a>');		
	}

	methods.destroy=function() {
		var theatre=this.data('theatre');
		if (theatre) { 
			clearInterval(theatre.interval);
			this.theatre('jump', 0);
			if (typeof theatre.effect.destroy == 'function') theatre.effect.destroy();
			this.removeClass('theatre-'+theatre.settings.effect.replace(/[^a-z0-9]+/ig, '-'));
			theatre.actors.each(function(){ // Restore original sizes
					var theatre=jQuery(this).data('theatre');
					jQuery(this).width(theatre.width);
					jQuery(this).height(theatre.height);
				})
		}
		jQuery('.theatre-control', this).remove();
	}

	methods.play=function() {
		var theatre=this.data('theatre');
		var stage=this;
		//methods.stop.apply(this);
		stage.theatre('next');
		theatre.interval=setInterval(function() {stage.theatre('next', true);}, theatre.settings.speed + theatre.settings.still);
	}

	methods.stop=function() {
		var theatre=this.data('theatre');
		clearInterval(theatre.interval);
		theatre.interval=false;
	}

	effects['fade']=
	effects['slide']=
	effects['show']=function(stage, actors, settings, theatre) {
		var x={
			fade: {show: 'fadeIn', hide: 'fadeOut'},
			slide: {show: 'slideDown', hide: 'slideUp'},
			show: {show: 'show', hide:'hide'}
		}[settings.effect];
		
		this.init=function() {
			actors[x.hide](0).first()[x.show](0);
			if (settings.effect == 'fade') actors.css({margin: 0, top: 0, left: 0, position: 'absolute'});
		}
		this.next=function() {
			actors.stop(true, true).css('z-index', 0)[x.hide](settings.speed)
				.eq(theatre.index).css('z-index', 10)[x.show](settings.speed);			
		}
		this.prev=function() {
			actors.stop(true, true).css('z-index', 0)[x.hide](settings.speed)
				.eq(theatre.index).css('z-index', 10)[x.show](settings.speed);						
		}
		this.destroy=function() {
			actors.stop(true, true).css({zIndex: '', top: '', left: '', position: '', margin: ''})[x.show](0);			
		}
	}

	effects['vertical']=	
	effects['horizontal']=function(stage, actors, settings) {
		var x={
			horizontal: {'size': 'outerWidth', 'direction': 'left'},
			vertical: {'size': 'outerHeight', 'direction': 'top'}
		}[settings.effect];

		this.init=function() {
			actors.fadeOut(0);
			this.align(0, 0);
			actors.fadeIn();			
		}
		this.next=function() {
			// jQuery(settings.selector, stage).last().stop(true, true); // to prevent visual contra-direction transition
			var curr=jQuery(settings.selector, stage).first();
			var offset=this.align(-curr[x.size]());
			curr.appendTo(stage);
		}
		this.prev=function() {
			var curr=jQuery(settings.selector, stage).last().prependTo(stage);
			curr.stop(true, true).css(x.direction, -curr[x.size]());
			this.align(0);
		}
		this.destroy=function() {
			actors.stop(true, true).css(x.direction, '').css({opacity: '', left: '', top: ''});
		}
		this.align=function(offset, speed) {
			var sacked=false;
			jQuery(settings.selector, stage).each(function() {
					var callBack=null
					if (offset < 0) {
						sacked=jQuery(this);
						callBack=function() {sacked.css(x.direction, offset);}
					}
					var props={};
					props[x.direction]=offset;
					jQuery(this).stop(true, true).animate(props, isNaN(speed) ? settings.speed : speed, callBack);
					offset+=jQuery(this)[x.size]();
				});
			return offset;
		}
	}

	effects['3d']=function(stage, actors, settings) {
		// Override custom/default settings
		settings.resize=false;
		
		var currIndex=0;
		var pivots=[];
		var maxWidth;
		var maxHeight;

		this.init=function() {
			var thisObj=this;
			maxWidth=stage.width() * 0.5;
			maxHeight=stage.height() * 0.8;
			actors.each(function(pos) {
					var rad=(2 * Math.PI * pos / actors.length) + Math.PI / 2;
					var x=Math.cos(rad);
					var y=Math.sin(rad);

					var margin=10;
					var sizeMin=0.2;
					var size=(y + 1) / 2 * (1 - sizeMin) + sizeMin;
					var x2=x * (stage.width() - margin) / 2 + stage.width() / 2;
					var y2=y * (stage.height() - margin) / 2 + stage.height() / 2;
					
					pivots.push({left: x2, top: y2, x: x, y: y, size: size, rad: rad});
				});
			this.animate();
			//console.log(pivots);
		}
		this.next=function() {
			if (++currIndex > actors.length - 1) currIndex=0;
			this.animate();
		}
		this.prev=function() {
			if (--currIndex < 0) currIndex=actors.length - 1;
			this.animate();			
		}
		this.destroy=function() {
			actors.stop(true, true).css({'z-index': '', opacity: '', left: '', top: ''});
		}
		this.animate=function() {
			var thisObj=this;
			actors.stop();
			actors.each(function(pos) {
					var pivot=pivots[(pos - currIndex + actors.length) % actors.length];
					var dim=thisObj.calcDim(jQuery(this), maxWidth, maxHeight, pivot.size);
					var left=Math.round(pivot.left - pivot.x * dim[0] / 2 - dim[0] / 2);
					var top=Math.round(pivot.top - pivot.y * dim[1] / 2 - dim[1] / 2);
					// use transitions where applicable for skew() or scale() http://www.alistapart.com/articles/understanding-css3-transitions/
					// https://github.com/brandonaaron/jquery-cssHooks/blob/master/boxreflect.js for JQuery 1.4.3
					jQuery(this).css({'z-index': Math.round(pivot.size * 1000)}).animate({opacity: pivot.size, left: left, top: top, width: dim[0], height: dim[1]}, settings.speed);
				});
		}
		this.calcDim=function(obj, maxWidth, maxHeight, size) {
			var dim=obj.data('theatre');
			var w=maxWidth;
			var h=dim.height / dim.width * w;
			if (h > maxHeight) {
				w=maxWidth * (maxHeight / h);
				h=maxHeight;
			}
			return [Math.round(w * size), Math.round(h * size)];
		}
	}
	
	
})(jQuery);
